@@ -215,7 +215,11 @@ enum class SkolemFunId
    */
   SETS_MAP_DOWN_ELEMENT,
   /** Higher-order type match predicate, see HoTermDb */
-  HO_TYPE_MATCH_PRED
+  HO_TYPE_MATCH_PRED,
+  /** the "none" term, for instantiation evaluation */
+  IEVAL_NONE,
+  /** the "some" term, for instantiation evaluation */
+  IEVAL_SOME
 };
 /** Converts a skolem function name to a string. */
 const char* toString(SkolemFunId id);
@@ -224,9 +228,19 @@ std::ostream& operator<<(std::ostream& out, SkolemFunId id);
 
 /**
  * A manager for skolems that can be used in proofs. This is designed to be
- * a trusted interface to NodeManager::mkSkolem, where one
- * must provide a definition for the skolem they create in terms of a
- * predicate that the introduced variable is intended to witness.
+ * a trusted interface for constructing variables of SKOLEM type, where one
+ * must provide information that characterizes the skolem. This information
+ * may either be:
+ * (1) the term that the skolem purifies (`mkPurifySkolem`)
+ * (2) a predicate the skolem satisfies (this is currently solely used by
+ * `mkSkolemize` for witnessing existential quantifiers),
+ * (3) an identifier (`mkSkolemFunction`), which are typically used for
+ * implementing theory-specific inferences that introduce symbols that
+ * are not interpreted by the theory (see SkolemFunId enum).
+ *
+ * If a variable cannot be associated with any of the above information,
+ * the method `mkDummySkolem` may be used, which always constructs a fresh
+ * skolem variable.
  *
  * It is implemented by mapping terms to an attribute corresponding to their
  * "original form" and "witness form" as described below. Hence, this class does
@@ -236,10 +250,10 @@ std::ostream& operator<<(std::ostream& out, SkolemFunId id);
  * We distinguish two kinds of mappings between terms and skolems:
  *
  * (1) "Original form", which associates skolems with the terms they purify.
- * This is used in mkPurifySkolem below.
+ * This is used in `mkPurifySkolem` below.
  *
- * (2) "Witness form", which associates skolems with their formal definition
- * as a witness term. This is used in mkSkolem below.
+ * (2) "Witness form", which associates skolems with a witness term whose
+ * body is a predicate they satisfy. This is used in `mkSkolemize` below.
  *
  * It is possible to unify these so that purification skolems for t are skolems
  * whose witness form is (witness ((x T)) (= x t)). However, there are
@@ -252,10 +266,7 @@ std::ostream& operator<<(std::ostream& out, SkolemFunId id);
  *
  * Furthermore, note that original form and witness form may share skolems
  * in the rare case that a witness term is purified. This is currently only the
- * case for algorithms that introduce witness, e.g. BV/set instantiation.
- *
- * Additionally, we consider a third class of skolems (mkSkolemFunction) which
- * are for convenience associated with an identifier, and not a witness term.
+ * case for algorithms that introduce witnesses, e.g. BV/set instantiation.
  */
 class SkolemManager
 {
@@ -281,58 +292,24 @@ class SkolemManager
     SKOLEM_ABSTRACT_VALUE = 4,
   };
   /**
-   * This makes a skolem of same type as bound variable v, (say its type is T),
-   * whose definition is (witness ((v T)) pred). This definition is maintained
-   * by this class.
+   * Make purification skolem. This skolem is unique for each t, which we
+   * implement via an attribute on t. This attribute is used to ensure to
+   * associate a unique skolem for each t.
    *
-   * Notice that (exists ((v T)) pred) should be a valid formula. This fact
-   * captures the reason for why the returned Skolem was introduced.
+   * Notice that a purification skolem is trivial to justify (via
+   * SKOLEM_INTRO), and hence it does not require a proof generator.
    *
-   * Take as an example extensionality in arrays:
-   *
-   * (declare-fun a () (Array Int Int))
-   * (declare-fun b () (Array Int Int))
-   * (assert (not (= a b)))
-   *
-   * To witness the index where the arrays a and b are disequal, it is intended
-   * we call this method on:
-   *   Node k = mkSkolem( x, F )
-   * where F is:
-   *   (=> (not (= a b)) (not (= (select a x) (select b x))))
-   * and x is a fresh bound variable of integer type. Internally, this will map
-   * k to the term:
-   *   (witness ((x Int)) (=> (not (= a b))
-   *                          (not (= (select a x) (select b x)))))
-   * A lemma generated by the array solver for extensionality may safely use
-   * the skolem k in the standard way:
-   *   (=> (not (= a b)) (not (= (select a k) (select b k))))
-   * Furthermore, notice that the following lemma does not involve fresh
-   * skolem variables and is valid according to the theory of arrays extended
-   * with support for witness:
-   *   (let ((w (witness ((x Int)) (=> (not (= a b))
-   *                                   (not (= (select a x) (select b x)))))))
-   *     (=> (not (= a b)) (not (= (select a w) (select b w)))))
-   * This version of the lemma, which requires no explicit tracking of free
-   * Skolem variables, can be obtained by a call to getWitnessForm(...)
-   * below. We call this the "witness form" of the lemma above.
-   *
-   * @param v The bound variable of the same type of the Skolem to create.
-   * @param pred The desired property of the Skolem to create, in terms of bound
-   * variable v.
-   * @param prefix The prefix of the name of the Skolem
-   * @param comment Debug information about the Skolem
-   * @param flags The flags for the Skolem (see SkolemFlags)
-   * @param pg The proof generator for this skolem. If non-null, this proof
-   * generator must respond to a call to getProofFor(exists v. pred) during
-   * the lifetime of the current node manager.
-   * @return The skolem whose witness form is registered by this class.
+   * Notice that we do not convert t to original form in this call. Thus,
+   * in very rare cases, two Skolems may be introduced that have the same
+   * original form. For example, let k be the skolem introduced to eliminate
+   * (ite A B C). Then, asking for the purify skolem for:
+   *  (ite (ite A B C) D E) and (ite k D E)
+   * will return two different Skolems.
    */
-  Node mkSkolem(Node v,
-                Node pred,
-                const std::string& prefix,
-                const std::string& comment = "",
-                int flags = SKOLEM_DEFAULT,
-                ProofGenerator* pg = nullptr);
+  Node mkPurifySkolem(Node t,
+                      const std::string& prefix,
+                      const std::string& comment = "",
+                      int flags = SKOLEM_DEFAULT);
   /**
    * Make skolemized form of existentially quantified formula q, and store its
    * Skolems into the argument skolems.
@@ -351,9 +328,8 @@ class SkolemManager
    * of w1 is expanded in the witness form of w2. This avoids variable
    * shadowing.
    *
-   * In contrast to mkSkolem, the proof generator is for the *entire*
-   * existentially quantified formula q, which may have multiple variables in
-   * its prefix.
+   * Notice that the proof generator is for the *entire* existentially
+   * quantified formula q, which may have multiple variables in its prefix.
    *
    * @param q The existentially quantified formula to skolemize,
    * @param skolems Vector to add Skolems of q to,
@@ -371,25 +347,6 @@ class SkolemManager
                    const std::string& comment = "",
                    int flags = SKOLEM_DEFAULT,
                    ProofGenerator* pg = nullptr);
-  /**
-   * Make purification skolem. This skolem is unique for each t, which we
-   * implement via an attribute on t. This attribute is used to ensure to
-   * associate a unique skolem for each t.
-   *
-   * Notice that a purification skolem is trivial to justify (via SKOLEM_INTRO),
-   * and hence it does not require a proof generator.
-   *
-   * Notice that we do not convert t to original form in this call. Thus,
-   * in very rare cases, two Skolems may be introduced that have the same
-   * original form. For example, let k be the skolem introduced to eliminate
-   * (ite A B C). Then, asking for the purify skolem for:
-   *  (ite (ite A B C) D E) and (ite k D E)
-   * will return two different Skolems.
-   */
-  Node mkPurifySkolem(Node t,
-                      const std::string& prefix,
-                      const std::string& comment = "",
-                      int flags = SKOLEM_DEFAULT);
   /**
    * Make skolem function. This method should be used for creating fixed
    * skolem functions of the forms described in SkolemFunId. The user of this
@@ -429,8 +386,8 @@ class SkolemManager
                         const std::vector<Node>& cacheVals,
                         int flags = SKOLEM_DEFAULT);
   /**
-   * Is k a skolem function? Returns true if k was generated by the above call.
-   * Updates the arguments to the values used when constructing it.
+   * Is k a skolem function? Returns true if k was generated by the above
+   * call. Updates the arguments to the values used when constructing it.
    */
   bool isSkolemFunction(Node k, SkolemFunId& id, Node& cacheVal) const;
   /**
@@ -457,7 +414,7 @@ class SkolemManager
                      int flags = SKOLEM_DEFAULT);
   /**
    * Get proof generator for existentially quantified formula q. This returns
-   * the proof generator that was provided in a call to mkSkolem above.
+   * the proof generator that was provided in a call to `mkSkolemize` above.
    */
   ProofGenerator* getProofGenerator(Node q) const;
 
@@ -474,8 +431,8 @@ class SkolemManager
    */
   static Node getWitnessForm(Node k);
   /**
-   * Convert to original form, which recursively replaces all skolems terms in n
-   * by the term they purify.
+   * Convert to original form, which recursively replaces all skolems terms in
+   * n by the term they purify.
    *
    * @param n The term or formula to convert to original form described above
    * @return n in original form.
@@ -484,9 +441,9 @@ class SkolemManager
   /**
    * Convert to unpurified form, which returns the term that k purifies. This
    * is literally the term that was passed as an argument to mkPurify on the
-   * call that created k. In contrast to getOriginalForm, this is not recursive
-   * w.r.t. skolems, so that the term purified by k may itself contain
-   * purification skolems that are not expanded.
+   * call that created k. In contrast to getOriginalForm, this is not
+   * recursive w.r.t. skolems, so that the term purified by k may itself
+   * contain purification skolems that are not expanded.
    *
    * @param k The skolem to convert to unpurified form
    * @return the unpurified form of k.
@@ -528,7 +485,8 @@ class SkolemManager
    *
    * This method additionally updates qskolem to be the skolemized form of q.
    * In the above example, this is set to:
-   *   (exists ((y Int)) (P (witness ((x Int)) (exists ((y' Int)) (P x y'))) y))
+   *   (exists
+   *       ((y Int)) (P (witness ((x Int)) (exists ((y' Int)) (P x y'))) y))
    */
   Node skolemize(Node q,
                  Node& qskolem,
@@ -546,6 +504,62 @@ class SkolemManager
                     const TypeNode& type,
                     const std::string& comment = "",
                     int flags = SKOLEM_DEFAULT);
+  /**
+   * This makes a skolem of same type as bound variable v, say its type is T,
+   * whose definition is (witness ((v T)) pred). This definition is maintained
+   * by this class.
+   *
+   * Notice that (exists ((v T)) pred) should be a valid formula. This fact
+   * captures the reason for why the returned Skolem was introduced.
+   *
+   * Note that arrays uses a skolem (which identifier ARRAY_DEQ_DIFF) via
+   * `mkSkolemFunction` for extensionality. However, consider the following
+   * example, which demonstrates how it could use a witness skolem for
+   * formalizing this instead:
+   *
+   * (declare-fun a () (Array Int Int))
+   * (declare-fun b () (Array Int Int))
+   * (assert (not (= a b)))
+   *
+   * To witness the index where the arrays a and b are disequal, it is intended
+   * we call this method on:
+   *   Node k = mkWitnessSkolem( x, F )
+   * where F is:
+   *   (=> (not (= a b)) (not (= (select a x) (select b x))))
+   * and x is a fresh bound variable of integer type. Internally, this will map
+   * k to the term:
+   *   (witness ((x Int)) (=> (not (= a b))
+   *                          (not (= (select a x) (select b x)))))
+   * A lemma generated by the array solver for extensionality may safely use
+   * the skolem k in the standard way:
+   *   (=> (not (= a b)) (not (= (select a k) (select b k))))
+   * Furthermore, notice that the following lemma does not involve fresh
+   * skolem variables and is valid according to the theory of arrays extended
+   * with support for witness:
+   *   (let ((w (witness ((x Int)) (=> (not (= a b))
+   *                                   (not (= (select a x) (select b x)))))))
+   *     (=> (not (= a b)) (not (= (select a w) (select b w)))))
+   * This version of the lemma, which requires no explicit tracking of free
+   * Skolem variables, can be obtained by a calls to getWitnessForm(...).
+   * We call this the "witness form" of the lemma above.
+   *
+   * @param v The bound variable of the same type of the Skolem to create.
+   * @param pred The desired property of the Skolem to create, in terms of bound
+   * variable v.
+   * @param prefix The prefix of the name of the Skolem
+   * @param comment Debug information about the Skolem
+   * @param flags The flags for the Skolem (see SkolemFlags)
+   * @param pg The proof generator for this skolem. If non-null, this proof
+   * generator must respond to a call to getProofFor(exists v. pred) during
+   * the lifetime of the current node manager.
+   * @return The skolem whose witness form is registered by this class.
+   */
+  Node mkWitnessSkolem(Node v,
+                       Node pred,
+                       const std::string& prefix,
+                       const std::string& comment = "",
+                       int flags = SKOLEM_DEFAULT,
+                       ProofGenerator* pg = nullptr);
 };
 
 }  // namespace cvc5::internal
