@@ -26,30 +26,39 @@
 #include "base/check.h"
 #include "base/output.h"
 #include "expr/kind.h"
+#include "parser/api/cpp/command.h"
 #include "parser/input.h"
 #include "parser/parser_exception.h"
-#include "smt/command.h"
+#include "parser/smt2/smt2_input.h"
 
 using namespace std;
 
 namespace cvc5 {
 namespace parser {
 
+class Parser::IncludeFileCache
+{
+ public:
+  IncludeFileCache() {}
+  ~IncludeFileCache()
+  {
+    for (size_t i = 0, isize = d_inCreated.size(); i < isize; i++)
+    {
+      d_inCreated[i]->free(d_inCreated[i]);
+    }
+  }
+  std::vector<pANTLR3_INPUT_STREAM> d_inCreated;
+};
+
 Parser::Parser(cvc5::Solver* solver,
                SymbolManager* sm,
-               bool strictMode,
-               bool parseOnly)
+               bool strictMode)
     : d_symman(sm),
       d_symtab(sm->getSymbolTable()),
-      d_assertionLevel(0),
-      d_anonymousFunctionCount(0),
       d_done(true),
       d_checksEnabled(true),
       d_strictMode(strictMode),
-      d_parseOnly(parseOnly),
       d_canIncludeFile(true),
-      d_logicIsForced(false),
-      d_forcedLogic(),
       d_solver(solver)
 {
 }
@@ -73,13 +82,11 @@ cvc5::Term Parser::getSymbol(const std::string& name, SymbolType type)
   // Functions share var namespace
   return d_symtab->lookup(name);
 }
-
-void Parser::forceLogic(const std::string& logic)
+const std::string& Parser::getForcedLogic() const
 {
-  Assert(!d_logicIsForced);
-  d_logicIsForced = true;
-  d_forcedLogic = logic;
+  return d_symman->getForcedLogic();
 }
+bool Parser::logicIsForced() const { return d_symman->isLogicForced(); }
 
 cvc5::Term Parser::getVariable(const std::string& name)
 {
@@ -163,8 +170,8 @@ cvc5::Sort Parser::getSort(const std::string& name)
   return t;
 }
 
-cvc5::Sort Parser::getSort(const std::string& name,
-                           const std::vector<cvc5::Sort>& params)
+cvc5::Sort Parser::getParametricSort(const std::string& name,
+                                     const std::vector<cvc5::Sort>& params)
 {
   checkDeclaration(name, CHECK_DECLARED, SYM_SORT);
   Assert(isDeclared(name, SYM_SORT));
@@ -172,18 +179,6 @@ cvc5::Sort Parser::getSort(const std::string& name,
   return t;
 }
 
-size_t Parser::getArity(const std::string& sort_name) {
-  checkDeclaration(sort_name, CHECK_DECLARED, SYM_SORT);
-  Assert(isDeclared(sort_name, SYM_SORT));
-  return d_symtab->lookupArity(sort_name);
-}
-
-/* Returns true if name is bound to a boolean variable. */
-bool Parser::isBoolean(const std::string& name) {
-  cvc5::Term expr = getVariable(name);
-  return !expr.isNull() && expr.getSort().isBoolean();
-}
-
 bool Parser::isFunctionLike(cvc5::Term fun)
 {
   if(fun.isNull()) {
@@ -194,12 +189,6 @@ bool Parser::isFunctionLike(cvc5::Term fun)
          || type.isDatatypeTester() || type.isDatatypeSelector();
 }
 
-/* Returns true if name is bound to a function returning boolean. */
-bool Parser::isPredicate(const std::string& name) {
-  cvc5::Term expr = getVariable(name);
-  return !expr.isNull() && expr.getSort().isPredicate();
-}
-
 cvc5::Term Parser::bindVar(const std::string& name,
                            const cvc5::Sort& type,
                            bool doOverload)
@@ -230,17 +219,6 @@ std::vector<cvc5::Term> Parser::bindBoundVars(
   return vars;
 }
 
-std::vector<cvc5::Term> Parser::bindVars(const std::vector<std::string> names,
-                                         const cvc5::Sort& type,
-                                         bool doOverload)
-{
-  std::vector<cvc5::Term> vars;
-  for (unsigned i = 0; i < names.size(); ++i) {
-    vars.push_back(bindVar(names[i], type, doOverload));
-  }
-  return vars;
-}
-
 std::vector<cvc5::Term> Parser::bindBoundVars(
     const std::vector<std::string> names, const cvc5::Sort& type)
 {
@@ -345,7 +323,7 @@ cvc5::Sort Parser::mkUnresolvedTypeConstructor(
   cvc5::Sort unresolved =
       d_solver->mkUnresolvedDatatypeSort(name, params.size());
   defineType(name, params, unresolved);
-  cvc5::Sort t = getSort(name, params);
+  cvc5::Sort t = getParametricSort(name, params);
   return unresolved;
 }
 
@@ -595,6 +573,8 @@ bool Parser::isDeclared(const std::string& name, SymbolType type) {
     case SYM_VARIABLE: return d_symtab->isBound(name);
     case SYM_SORT:
       return d_symtab->isBoundType(name);
+    case SYM_VERBATIM:
+      Unreachable();
   }
   Assert(false);  // Unhandled(type);
   return false;
@@ -904,17 +884,6 @@ std::wstring Parser::processAdHocStringEsc(const std::string& s)
   return res;
 }
 
-cvc5::Term Parser::mkStringConstant(const std::string& s)
-{
-  if (d_solver->getOption("input-language") == "LANG_SMTLIB_V2_6")
-  {
-    return d_solver->mkString(s, true);
-  }
-  // otherwise, we must process ad-hoc escape sequences
-  std::wstring str = processAdHocStringEsc(s);
-  return d_solver->mkString(str);
-}
-
 cvc5::Term Parser::mkCharConstant(const std::string& s)
 {
   Assert(s.find_first_not_of("0123456789abcdefABCDEF", 0) == std::string::npos
@@ -924,5 +893,142 @@ cvc5::Term Parser::mkCharConstant(const std::string& s)
   return d_solver->mkString(std::wstring(1, val));
 }
 
+/* The includes are managed in the lexer but called in the parser */
+// Inspired by http://www.antlr3.org/api/C/interop.html
+
+bool newInputStream(std::string fileName,
+                    pANTLR3_LEXER lexer,
+                    std::vector<pANTLR3_INPUT_STREAM>& inc)
+{
+  Trace("parser") << "Including " << fileName << std::endl;
+  // Create a new input stream and take advantage of built in stream stacking
+  // in C target runtime.
+  //
+  pANTLR3_INPUT_STREAM in;
+#ifdef CVC5_ANTLR3_OLD_INPUT_STREAM
+  in = antlr3AsciiFileStreamNew((pANTLR3_UINT8)fileName.c_str());
+#else  /* CVC5_ANTLR3_OLD_INPUT_STREAM */
+  in = antlr3FileStreamNew((pANTLR3_UINT8)fileName.c_str(), ANTLR3_ENC_8BIT);
+#endif /* CVC5_ANTLR3_OLD_INPUT_STREAM */
+  if (in == NULL)
+  {
+    Trace("parser") << "Can't open " << fileName << std::endl;
+    return false;
+  }
+  // Same thing as the predefined PUSHSTREAM(in);
+  lexer->pushCharStream(lexer, in);
+  // restart it
+  // lexer->rec->state->tokenStartCharIndex  = -10;
+  // lexer->emit(lexer);
+
+  // Note that the input stream is not closed when it EOFs, I don't bother
+  // to do it here, but it is up to you to track streams created like this
+  // and destroy them when the whole parse session is complete. Remember that
+  // you don't want to do this until all tokens have been manipulated all the
+  // way through your tree parsers etc as the token does not store the text it
+  // just refers back to the input stream and trying to get the text for it will
+  // abort if you close the input stream too early.
+  //
+  inc.push_back(in);
+
+  // TODO what said before
+  return true;
+}
+
+Parser::IncludeFileCache* Parser::getIncludeFileCache()
+{
+  if (d_incCache == nullptr)
+  {
+    d_incCache.reset(new IncludeFileCache);
+  }
+  return d_incCache.get();
+}
+
+void Parser::includeSmt2File(const std::string& filename)
+{
+  // security for online version
+  if (!canIncludeFile())
+  {
+    parseError("include-file feature was disabled for this run.");
+  }
+
+  // Get the lexer
+  AntlrInput* ai = static_cast<AntlrInput*>(getInput());
+  pANTLR3_LEXER lexer = ai->getAntlr3Lexer();
+  // get the name of the current stream "Does it work inside an include?"
+  const std::string inputName = ai->getInputStreamName();
+
+  // Find the directory of the current input file
+  std::string path;
+  size_t pos = inputName.rfind('/');
+  if (pos != std::string::npos)
+  {
+    path = std::string(inputName, 0, pos + 1);
+  }
+  path.append(filename);
+  IncludeFileCache* ifc = getIncludeFileCache();
+  if (!newInputStream(path, lexer, ifc->d_inCreated))
+  {
+    parseError("Couldn't open include file `" + path + "'");
+  }
+}
+
+void Parser::includeTptpFile(const std::string& fileName,
+                             const std::string& tptpDir)
+{
+  // security for online version
+  if (!canIncludeFile())
+  {
+    parseError("include-file feature was disabled for this run.");
+  }
+
+  // Get the lexer
+  AntlrInput* ai = static_cast<AntlrInput*>(getInput());
+  pANTLR3_LEXER lexer = ai->getAntlr3Lexer();
+
+  // push the inclusion scope; will be popped by our special popCharStream
+  // would be necessary for handling symbol filtering in inclusions
+  // pushScope();
+
+  // get the name of the current stream "Does it work inside an include?"
+  const std::string inputName = ai->getInputStreamName();
+
+  // Test in the directory of the actual parsed file
+  std::string currentDirFileName;
+  if (inputName != "<stdin>")
+  {
+    // TODO: Use dirname or Boost::filesystem?
+    size_t pos = inputName.rfind('/');
+    if (pos != std::string::npos)
+    {
+      currentDirFileName = std::string(inputName, 0, pos + 1);
+    }
+    currentDirFileName.append(fileName);
+    IncludeFileCache* ifc = getIncludeFileCache();
+    if (newInputStream(currentDirFileName, lexer, ifc->d_inCreated))
+    {
+      return;
+    }
+  }
+  else
+  {
+    currentDirFileName = "<unknown current directory for stdin>";
+  }
+
+  if (tptpDir.empty())
+  {
+    parseError("Couldn't open included file: " + fileName
+               + " at " + currentDirFileName + " and the TPTP directory is not specified (environment variable TPTP)");
+  };
+
+  std::string tptpDirFileName = tptpDir + fileName;
+  IncludeFileCache* ifc = getIncludeFileCache();
+  if (!newInputStream(tptpDirFileName, lexer, ifc->d_inCreated))
+  {
+    parseError("Couldn't open included file: " + fileName + " at "
+               + currentDirFileName + " or " + tptpDirFileName);
+  }
+}
+
 }  // namespace parser
 }  // namespace cvc5
