@@ -21,8 +21,8 @@
 
 #include "api/cpp/cvc5.h"
 #include "base/check.h"
+#include "parser/api/cpp/command.h"
 #include "parser/parser.h"
-#include "smt/command.h"
 #include "theory/logic_info.h"
 
 // ANTLR defines these, which is really bad!
@@ -34,9 +34,8 @@ namespace parser {
 
 Tptp::Tptp(cvc5::Solver* solver,
            SymbolManager* sm,
-           bool strictMode,
-           bool parseOnly)
-    : Parser(solver, sm, strictMode, parseOnly),
+           bool strictMode)
+    : Parser(solver, sm, strictMode),
       d_cnf(false),
       d_fof(false),
       d_hol(false)
@@ -66,12 +65,14 @@ Tptp::Tptp(cvc5::Solver* solver,
     }
   }
   d_hasConjecture = false;
+  // Handle forced logic immediately.
+  if (sm->isLogicForced())
+  {
+    preemptCommand(new SetBenchmarkLogicCommand(sm->getForcedLogic()));
+  }
 }
 
 Tptp::~Tptp() {
-  for( unsigned i=0; i<d_in_created.size(); i++ ){
-    d_in_created[i]->free(d_in_created[i]);
-  }
 }
 
 void Tptp::addTheory(Theory theory) {
@@ -105,90 +106,6 @@ void Tptp::addTheory(Theory theory) {
   }
 }
 
-
-/* The include are managed in the lexer but called in the parser */
-// Inspired by http://www.antlr3.org/api/C/interop.html
-
-bool newInputStream(std::string fileName, pANTLR3_LEXER lexer, std::vector< pANTLR3_INPUT_STREAM >& inc ) {
-  Trace("parser") << "Including " << fileName << std::endl;
-  // Create a new input stream and take advantage of built in stream stacking
-  // in C target runtime.
-  //
-  pANTLR3_INPUT_STREAM    in;
-#ifdef CVC5_ANTLR3_OLD_INPUT_STREAM
-  in = antlr3AsciiFileStreamNew((pANTLR3_UINT8) fileName.c_str());
-#else  /* CVC5_ANTLR3_OLD_INPUT_STREAM */
-  in = antlr3FileStreamNew((pANTLR3_UINT8) fileName.c_str(), ANTLR3_ENC_8BIT);
-#endif /* CVC5_ANTLR3_OLD_INPUT_STREAM */
-  if(in == NULL) {
-    Trace("parser") << "Can't open " << fileName << std::endl;
-    return false;
-  }
-  // Same thing as the predefined PUSHSTREAM(in);
-  lexer->pushCharStream(lexer,in);
-  // restart it
-  //lexer->rec->state->tokenStartCharIndex  = -10;
-  //lexer->emit(lexer);
-
-  // Note that the input stream is not closed when it EOFs, I don't bother
-  // to do it here, but it is up to you to track streams created like this
-  // and destroy them when the whole parse session is complete. Remember that you
-  // don't want to do this until all tokens have been manipulated all the way through
-  // your tree parsers etc as the token does not store the text it just refers
-  // back to the input stream and trying to get the text for it will abort if you
-  // close the input stream too early.
-  //
-  inc.push_back( in );
-
-  //TODO what said before
-  return true;
-}
-
-void Tptp::includeFile(std::string fileName) {
-  // security for online version
-  if(!canIncludeFile()) {
-    parseError("include-file feature was disabled for this run.");
-  }
-
-  // Get the lexer
-  AntlrInput * ai = static_cast<AntlrInput *>(getInput());
-  pANTLR3_LEXER lexer = ai->getAntlr3Lexer();
-
-  // push the inclusion scope; will be popped by our special popCharStream
-  // would be necessary for handling symbol filtering in inclusions
-  //pushScope();
-
-  // get the name of the current stream "Does it work inside an include?"
-  const std::string inputName = ai->getInputStreamName();
-
-  // Test in the directory of the actual parsed file
-  std::string currentDirFileName;
-  if(inputName != "<stdin>") {
-    // TODO: Use dirname or Boost::filesystem?
-    size_t pos = inputName.rfind('/');
-    if(pos != std::string::npos) {
-      currentDirFileName = std::string(inputName, 0, pos + 1);
-    }
-    currentDirFileName.append(fileName);
-    if(newInputStream(currentDirFileName,lexer, d_in_created)) {
-      return;
-    }
-  } else {
-    currentDirFileName = "<unknown current directory for stdin>";
-  }
-
-  if(d_tptpDir.empty()) {
-    parseError("Couldn't open included file: " + fileName
-               + " at " + currentDirFileName + " and the TPTP directory is not specified (environment variable TPTP)");
-  };
-
-  std::string tptpDirFileName = d_tptpDir + fileName;
-  if(! newInputStream(tptpDirFileName,lexer, d_in_created)) {
-    parseError("Couldn't open included file: " + fileName
-               + " at " + currentDirFileName + " or " + tptpDirFileName);
-  }
-}
-
 void Tptp::checkLetBinding(const std::vector<cvc5::Term>& bvlist,
                            cvc5::Term lhs,
                            cvc5::Term rhs,
@@ -493,6 +410,8 @@ cvc5::Term Tptp::mkDecimal(
   return d_solver->mkReal(ss.str());
 }
 
+const std::string& Tptp::getTptpDir() const { return d_tptpDir; }
+
 bool Tptp::hol() const { return d_hol; }
 void Tptp::setHol()
 {
@@ -504,12 +423,6 @@ void Tptp::setHol()
   d_solver->setLogic("HO_UF");
 }
 
-void Tptp::forceLogic(const std::string& logic)
-{
-  Parser::forceLogic(logic);
-  preemptCommand(new SetBenchmarkLogicCommand(logic));
-}
-
 void Tptp::addFreeVar(cvc5::Term var)
 {
   Assert(cnf());
