@@ -191,8 +191,6 @@ void MipLibTrick::collectBooleanVariables(
 PreprocessingPassResult MipLibTrick::applyInternal(
     AssertionPipeline* assertionsToPreprocess)
 {
-  Assert(assertionsToPreprocess->getRealAssertionsEnd()
-         == assertionsToPreprocess->size());
   Assert(!options().base.incrementalSolving);
 
   // collect Boolean variables
@@ -615,9 +613,7 @@ PreprocessingPassResult MipLibTrick::applyInternal(
   if (!removeAssertions.empty())
   {
     Trace("miplib") << " scrubbing miplib encoding..." << endl;
-    for (size_t i = 0, size = assertionsToPreprocess->getRealAssertionsEnd();
-         i < size;
-         ++i)
+    for (size_t i = 0, size = assertionsToPreprocess->size(); i < size; ++i)
     {
       Node assertion = (*assertionsToPreprocess)[i];
       if (removeAssertions.find(assertion.getId()) != removeAssertions.end())
@@ -646,7 +642,6 @@ PreprocessingPassResult MipLibTrick::applyInternal(
   {
     Trace("miplib") << " miplib pass found nothing." << endl;
   }
-  assertionsToPreprocess->updateRealAssertionsEnd();
   return PreprocessingPassResult::NO_CONFLICT;
 }
 
