@@ -31,6 +31,23 @@ namespace cvc5::internal {
 
 namespace proof {
 
+std::unordered_map<Kind, AletheRule> s_bvKindToAletheRule = {
+  {kind::BITVECTOR_ULT, AletheRule::BV_BITBLAST_STEP_BVULT},
+  {kind::VARIABLE, AletheRule::BV_BITBLAST_STEP_VAR},
+  {kind::BITVECTOR_AND, AletheRule::BV_BITBLAST_STEP_BVAND},
+  {kind::BITVECTOR_OR, AletheRule::BV_BITBLAST_STEP_BVOR},
+  {kind::BITVECTOR_XOR, AletheRule::BV_BITBLAST_STEP_BVXOR},
+  {kind::BITVECTOR_XNOR, AletheRule::BV_BITBLAST_STEP_BVXNOR},
+  {kind::BITVECTOR_NOT, AletheRule::BV_BITBLAST_STEP_BVNOT},
+  {kind::BITVECTOR_ADD, AletheRule::BV_BITBLAST_STEP_BVADD},
+  {kind::BITVECTOR_NEG, AletheRule::BV_BITBLAST_STEP_BVNEG},
+  {kind::BITVECTOR_MULT, AletheRule::BV_BITBLAST_STEP_BVMULT},
+  {kind::BITVECTOR_CONCAT, AletheRule::BV_BITBLAST_STEP_CONCAT},
+  {kind::CONST_BITVECTOR, AletheRule::BV_BITBLAST_STEP_CONST},
+  {kind::BITVECTOR_EXTRACT, AletheRule::BV_BITBLAST_STEP_EXTRACT},
+  {kind::EQUAL, AletheRule::BV_BITBLAST_STEP_BVEQUAL},
+};
+
 AletheProofPostprocessCallback::AletheProofPostprocessCallback(
     Env& env, AletheNodeConverter& anc, bool resPivots)
     : EnvObj(env), d_anc(anc), d_resPivots(resPivots)
@@ -230,7 +247,7 @@ bool AletheProofPostprocessCallback::update(Node res,
       // Build vp1
       std::vector<Node> negNode{d_cl};
       std::vector<Node> sanitized_args;
-      for (Node arg : args)
+      for (const Node& arg : args)
       {
         negNode.push_back(arg.notNode());  // (not F1) ... (not Fn)
         sanitized_args.push_back(d_anc.convert(arg));
@@ -1128,6 +1145,22 @@ bool AletheProofPostprocessCallback::update(Node res,
                               {},
                               *cdp);
     }
+    // ======== Bitvector
+    //
+    // ------------------------ BV_BITBLAST_STEP_BV<KIND>
+    //  (cl (= t bitblast(t)))
+    case PfRule::BV_BITBLAST_STEP:
+    {
+      Assert(s_bvKindToAletheRule.find(res[0].getKind())
+             != s_bvKindToAletheRule.end())
+          << "Bit-blasted kind not supported in Alethe post-processing.";
+      return addAletheStep(s_bvKindToAletheRule.at(res[0].getKind()),
+                           res,
+                           nm->mkNode(kind::SEXPR, d_cl, res),
+                           children,
+                           {},
+                           *cdp);
+    }
     //================================================= Quantifiers rules
     // ======== Instantiate
     // See proof_rule.h for documentation on the INSTANTIATE rule. This
@@ -1205,7 +1238,7 @@ bool AletheProofPostprocessCallback::update(Node res,
     // * the corresponding proof node is (<= i greatestIntLessThan(c))
     case PfRule::INT_TIGHT_UB:
     {
-      Node vp1 = nm->mkNode(kind::SEXPR, d_cl, children[0], res);
+      Node vp1 = nm->mkNode(kind::SEXPR, d_cl, children[0].notNode(), res);
       std::vector<Node> new_children = {vp1, children[0]};
       new_args.push_back(nm->mkConstInt(Rational(1)));
       return addAletheStep(AletheRule::LA_GENERIC, vp1, vp1, {}, new_args, *cdp)
@@ -1228,7 +1261,7 @@ bool AletheProofPostprocessCallback::update(Node res,
     // * the corresponding proof node is (>= i leastIntGreaterThan(c))
     case PfRule::INT_TIGHT_LB:
     {
-      Node vp1 = nm->mkNode(kind::SEXPR, d_cl, children[0], res);
+      Node vp1 = nm->mkNode(kind::SEXPR, d_cl, children[0].notNode(), res);
       std::vector<Node> new_children = {vp1, children[0]};
       new_args.push_back(nm->mkConstInt(Rational(1)));
       return addAletheStep(AletheRule::LA_GENERIC, vp1, vp1, {}, new_args, *cdp)
