@@ -22,6 +22,7 @@
 
 #include "expr/node.h"
 #include "smt/env.h"
+#include "theory/arith/arith_subs.h"
 
 namespace cvc5::internal {
 namespace theory {
@@ -29,7 +30,7 @@ namespace arith {
 
 /**
  * Check if the expression `expr` is zero over the given model.
- * The model { nodes -> repls } may contain real algebraic numbers in standard
+ * The model subs may contain real algebraic numbers in standard
  * witness form. The environment is used for rewriting.
  *
  * The result is true or false, if the expression could be evaluated. If it
@@ -38,8 +39,7 @@ namespace arith {
  */
 std::optional<bool> isExpressionZero(Env& env,
                                      Node expr,
-                                     const std::vector<TNode>& nodes,
-                                     const std::vector<TNode>& repls);
+                                     const ArithSubs& subs);
 }
 }  // namespace theory
 }  // namespace cvc5::internal
