@@ -368,18 +368,19 @@ Node OperatorElim::eliminateOperators(Node node,
           }
         }
 
-        Kind rk =
-            k == ARCSINE
-                ? SINE
-                : (k == ARCCOSINE
-                       ? COSINE
-                       : (k == ARCTANGENT
-                              ? TANGENT
-                              : (k == ARCCOSECANT
-                                     ? COSECANT
-                                     : (k == ARCSECANT ? SECANT : COTANGENT))));
+        Kind rk;
+        switch (k)
+        {
+          case ARCSINE: rk = SINE; break;
+          case ARCCOSINE: rk = COSINE; break;
+          case ARCTANGENT: rk = TANGENT; break;
+          case ARCCOSECANT: rk = COSECANT; break;
+          case ARCSECANT: rk = SECANT; break;
+          case ARCCOTANGENT: rk = COTANGENT; break;
+          default: Unreachable() << "Unexpected kind " << k;
+        }
         Node invTerm = nm->mkNode(rk, var);
-        lem = nm->mkNode(AND, rlem, invTerm.eqNode(node[0]));
+        lem = nm->mkNode(AND, rlem, mkEquality(invTerm, node[0]));
         if (!cond.isNull())
         {
           lem = nm->mkNode(IMPLIES, cond, lem);
@@ -390,8 +391,6 @@ Node OperatorElim::eliminateOperators(Node node,
       return var;
     }
 
-    case BITVECTOR_TO_NAT: return eliminateBv2Nat(node); break;
-    case INT_TO_BITVECTOR: return eliminateInt2Bv(node); break;
     default: break;
   }
   return node;
