@@ -15,13 +15,13 @@
 
 #include "cvc5_private.h"
 
-#pragma once
+#ifndef CVC5__THEORY__BV__THEORY_BV_REWRITE_RULES_H
+#define CVC5__THEORY__BV__THEORY_BV_REWRITE_RULES_H
 
 #include <sstream>
 
 #include "context/context.h"
 #include "printer/printer.h"
-#include "smt/solver_engine.h"
 #include "theory/bv/theory_bv_utils.h"
 #include "theory/theory.h"
 #include "util/statistics_stats.h"
@@ -73,6 +73,13 @@ enum RewriteRuleId
   SremEliminateFewerBitwiseOps,
   ZeroExtendEliminate,
   SignExtendEliminate,
+  UaddoEliminate,
+  SaddoEliminate,
+  UmuloEliminate,
+  SmuloEliminate,
+  UsuboEliminate,
+  SsuboEliminate,
+  SdivoEliminate,
   BVToNatEliminate,
   IntToBVEliminate,
 
@@ -748,12 +755,12 @@ struct FixpointRewriteStrategy {
       if (R19::applies(current)) current = R19::template run<false>(current);
       if (R20::applies(current)) current = R20::template run<false>(current);
     } while (previous != current);
-    
+
     return current;
   }
 };
 
-
-} // End namespace bv
-} // End namespace theory
-}  // End namespace cvc5::internal
+}  // namespace bv
+}  // namespace theory
+}  // namespace cvc5::internal
+#endif
