@@ -269,6 +269,8 @@ const char* toString(InferenceId i)
       return "QUANTIFIERS_SYGUS_STREAM_EXCLUDE_CURRENT";
     case InferenceId::QUANTIFIERS_SYGUS_EXAMPLE_INFER_CONTRA:
       return "QUANTIFIERS_SYGUS_EXAMPLE_INFER_CONTRA";
+    case InferenceId::QUANTIFIERS_SYGUS_SI_INFEASIBLE:
+      return "QUANTIFIERS_SYGUS_SI_INFEASIBLE";
     case InferenceId::QUANTIFIERS_SYGUS_UNIF_PI_INTER_ENUM_SB:
       return "QUANTIFIERS_SYGUS_UNIF_PI_INTER_ENUM_SB";
     case InferenceId::QUANTIFIERS_SYGUS_UNIF_PI_SEPARATION:
@@ -303,6 +305,8 @@ const char* toString(InferenceId i)
       return "QUANTIFIERS_SYGUS_PBE_EXCLUDE";
     case InferenceId::QUANTIFIERS_SYGUS_PBE_CONSTRUCT_SOL:
       return "QUANTIFIERS_SYGUS_PBE_CONSTRUCT_SOL";
+    case InferenceId::QUANTIFIERS_SYGUS_COMPLETE_ENUM:
+      return "QUANTIFIERS_SYGUS_COMPLETE_ENUM";
     case InferenceId::QUANTIFIERS_DSPLIT: return "QUANTIFIERS_DSPLIT";
     case InferenceId::QUANTIFIERS_CONJ_GEN_SPLIT:
       return "QUANTIFIERS_CONJ_GEN_SPLIT";
@@ -521,6 +525,8 @@ const char* toString(InferenceId i)
       return "UF_HO_MODEL_EXTENSIONALITY";
     case InferenceId::UF_HO_LAMBDA_UNIV_EQ: return "HO_LAMBDA_UNIV_EQ";
     case InferenceId::UF_HO_LAMBDA_APP_REDUCE: return "HO_LAMBDA_APP_REDUCE";
+    case InferenceId::UF_ARITH_BV_CONV_REDUCTION:
+      return "UF_ARITH_BV_CONV_REDUCTION";
 
     default: return "?";
   }
