@@ -222,7 +222,11 @@ void TheorySetsPrivate::fullEffortCheck()
     Trace("sets") << "...iterate full effort check..." << std::endl;
     fullEffortReset();
 
-    Trace("sets-eqc") << "Equality Engine:" << std::endl;
+    if (TraceIsOn("sets-eqc"))
+    {
+      Trace("sets-eqc") << "Equality Engine:" << std::endl;
+      Trace("sets-eqc") << d_equalityEngine->debugPrintEqc() << std::endl;
+    }
     std::map<TypeNode, unsigned> eqcTypeCount;
     eq::EqClassesIterator eqcs_i = eq::EqClassesIterator(d_equalityEngine);
     while (!eqcs_i.isFinished())
@@ -231,22 +235,10 @@ void TheorySetsPrivate::fullEffortCheck()
       TypeNode tn = eqc.getType();
       d_state.registerEqc(tn, eqc);
       eqcTypeCount[tn]++;
-      Trace("sets-eqc") << "[" << eqc << "] : ";
       eq::EqClassIterator eqc_i = eq::EqClassIterator(eqc, d_equalityEngine);
       while (!eqc_i.isFinished())
       {
         Node n = (*eqc_i);
-        if (n != eqc)
-        {
-          if (TraceIsOn("sets-eqc"))
-          {
-            Trace("sets-eqc") << n;
-            if (n.isConst())
-            {
-              Trace("sets-eqc") << " (const) ";
-            }
-          }
-        }
         TypeNode tnn = n.getType();
         // register it with the state
         d_state.registerTerm(eqc, tnn, n);
@@ -295,12 +287,9 @@ void TheorySetsPrivate::fullEffortCheck()
         }
         ++eqc_i;
       }
-      Trace("sets-eqc") << std::endl;
       ++eqcs_i;
     }
 
-    Trace("sets-eqc") << "...finished equality engine." << std::endl;
-
     if (TraceIsOn("sets-state"))
     {
       Trace("sets-state") << "Equivalence class counters:" << std::endl;
@@ -1355,7 +1344,7 @@ void TheorySetsPrivate::postCheck(Theory::Effort level)
         if (!d_state.isInConflict() && !d_im.hasSentLemma()
             && d_fullCheckIncomplete)
         {
-          d_im.setIncomplete(d_fullCheckIncompleteId);
+          d_im.setModelUnsound(d_fullCheckIncompleteId);
         }
       }
     }
