@@ -128,8 +128,9 @@ class Theory : protected EnvObj
    */
   virtual void processCarePairArgs(TNode a, TNode b);
   /**
-   * Are care disequal? Return true if x and y are shared terms that are
-   * disequal according to the valuation.
+   * Are care disequal? Return true if x and y are distinct constants, shared
+   * terms that are disequal according to the valuation, or otherwise
+   * disequal according to the equality engine of this theory.
    */
   virtual bool areCareDisequal(TNode x, TNode y);
 
@@ -560,14 +561,21 @@ class Theory : protected EnvObj
    *
    * @param termSet The set to add terms to
    * @param includeShared Whether to include the shared terms of the theory
+   * @param irrKind The kinds
    */
   void collectAssertedTerms(std::set<Node>& termSet,
-                            bool includeShared = true) const;
+                            bool includeShared,
+                            const std::set<Kind>& irrKinds) const;
+  /** Same as above, using the irrelevant model kinds for irrKinds.*/
+  void collectAssertedTermsForModel(std::set<Node>& termSet,
+                                    bool includeShared = true) const;
   /**
    * Helper function for collectAssertedTerms, adds all subterms
    * belonging to this theory to termSet.
    */
-  void collectTerms(TNode n, std::set<Node>& termSet) const;
+  void collectTerms(TNode n,
+                    std::set<Node>& termSet,
+                    const std::set<Kind>& irrKinds) const;
   /**
    * Collect model values, after equality information is added to the model.
    * The argument termSet is the set of relevant terms returned by
