@@ -27,6 +27,7 @@
 #include "options/smt_options.h"
 #include "options/theory_options.h"
 #include "printer/printer.h"
+#include "smt/solver_engine_state.h"
 #include "proof/lazy_proof.h"
 #include "proof/proof_checker.h"
 #include "proof/proof_ensure_closed.h"
@@ -72,6 +73,7 @@ namespace theory {
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_UF)        \
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_ARITH)     \
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_BV)        \
+  CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_FF)        \
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_FP)        \
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_ARRAYS)    \
   CVC5_FOR_EACH_THEORY_STATEMENT(cvc5::internal::theory::THEORY_DATATYPES) \
@@ -225,17 +227,18 @@ TheoryEngine::TheoryEngine(Env& env)
       d_decManager(new DecisionManager(userContext())),
       d_relManager(nullptr),
       d_inConflict(context(), false),
-      d_inSatMode(false),
-      d_incomplete(context(), false),
-      d_incompleteTheory(context(), THEORY_BUILTIN),
-      d_incompleteId(context(), IncompleteId::UNKNOWN),
+      d_modelUnsound(context(), false),
+      d_modelUnsoundTheory(context(), THEORY_BUILTIN),
+      d_modelUnsoundId(context(), IncompleteId::UNKNOWN),
+      d_refutationUnsound(userContext(), false),
+      d_refutationUnsoundTheory(userContext(), THEORY_BUILTIN),
+      d_refutationUnsoundId(userContext(), IncompleteId::UNKNOWN),
       d_propagationMap(context()),
       d_propagationMapTimestamp(context(), 0),
       d_propagatedLiterals(context()),
       d_propagatedLiteralsIndex(context(), 0),
       d_atomRequests(context()),
-      d_combineTheoriesTime(statisticsRegistry().registerTimer(
-          "TheoryEngine::combineTheoriesTime")),
+      d_stats(statisticsRegistry()),
       d_true(),
       d_false(),
       d_interrupted(false),
@@ -440,25 +443,38 @@ void TheoryEngine::check(Theory::Effort effort) {
       // We are still satisfiable, propagate as much as possible
       propagate(effort);
 
-      // We do combination if all has been processed and we are in fullcheck
-      if (Theory::fullEffort(effort) && logicInfo().isSharingEnabled()
-          && !d_factsAsserted && !needCheck() && !d_inConflict)
+      if (Theory::fullEffort(effort))
       {
-        // Do the combination
-        Trace("theory") << "TheoryEngine::check(" << effort << "): running combination" << endl;
+        d_stats.d_fullEffortChecks++;
+        // We do combination if all has been processed and we are in fullcheck
+        if (logicInfo().isSharingEnabled() && !d_factsAsserted && !needCheck()
+            && !d_inConflict)
         {
-          TimerStat::CodeTimer combineTheoriesTimer(d_combineTheoriesTime);
-          d_tc->combineTheories();
-        }
-        if (logicInfo().isQuantified())
-        {
-          d_quantEngine->notifyCombineTheories();
+          d_stats.d_combineTheoriesCalls++;
+          // Do the combination
+          Trace("theory") << "TheoryEngine::check(" << effort
+                          << "): running combination" << endl;
+          {
+            TimerStat::CodeTimer combineTheoriesTimer(
+                d_stats.d_combineTheoriesTime);
+            d_tc->combineTheories();
+          }
+          if (logicInfo().isQuantified())
+          {
+            d_quantEngine->notifyCombineTheories();
+          }
         }
       }
+      else
+      {
+        Assert(effort == Theory::EFFORT_STANDARD);
+        d_stats.d_stdEffortChecks++;
+      }
     }
 
     // Must consult quantifiers theory for last call to ensure sat, or otherwise add a lemma
     if( Theory::fullEffort(effort) && ! d_inConflict && ! needCheck() ) {
+      d_stats.d_lcEffortChecks++;
       Trace("theory::assertions-model") << endl;
       if (TraceIsOn("theory::assertions-model")) {
         printAssertions("theory::assertions-model");
@@ -494,12 +510,6 @@ void TheoryEngine::check(Theory::Effort effort) {
       {
         d_relManager->notifyCandidateModel(getModel());
       }
-      if (!d_inConflict && !needCheck())
-      {
-        // We only mark that we are in "SAT mode". We build the model later only
-        // if the user asks for it via getBuiltModel.
-        d_inSatMode = true;
-      }
     }
 
     Trace("theory") << "TheoryEngine::check(" << effort << "): done, we are " << (d_inConflict ? "unsat" : "sat") << (d_lemmasAdded ? " with new lemmas" : " with no new lemmas");
@@ -515,7 +525,7 @@ void TheoryEngine::check(Theory::Effort effort) {
       {
         // Do post-processing of model from the theories (e.g. used for
         // THEORY_SEP to construct heap model)
-        d_tc->postProcessModel(d_incomplete.get());
+        d_tc->postProcessModel(d_modelUnsound.get());
       }
     }
   } catch(const theory::Interrupted&) {
@@ -606,15 +616,9 @@ TheoryModel* TheoryEngine::getModel()
 TheoryModel* TheoryEngine::getBuiltModel()
 {
   Assert(d_tc != nullptr);
-  // If this method was called, we should be in SAT mode, and produceModels
-  // should be true.
+  // If this method was called, produceModels should be true.
   AlwaysAssert(options().smt.produceModels);
-  if (!d_inSatMode)
-  {
-    // not available, perhaps due to interuption.
-    return nullptr;
-  }
-  // must build model at this point
+  // we must build model at this point
   if (!d_tc->buildModel())
   {
     return nullptr;
@@ -679,8 +683,6 @@ bool TheoryEngine::presolve() {
 }/* TheoryEngine::presolve() */
 
 void TheoryEngine::postsolve() {
-  // no longer in SAT mode
-  d_inSatMode = false;
   // Reset the interrupt flag
   d_interrupted = false;
 }
@@ -744,17 +746,10 @@ theory::Theory::PPAssertStatus TheoryEngine::solve(
   TNode atom = literal.getKind() == kind::NOT ? literal[0] : literal;
   Trace("theory::solve") << "TheoryEngine::solve(" << literal << "): solving with " << theoryOf(atom)->getId() << endl;
 
-  theory::TheoryId tid = d_env.theoryOf(atom);
-  if (!isTheoryEnabled(tid) && tid != THEORY_SAT_SOLVER)
-  {
-    stringstream ss;
-    ss << "The logic was specified as " << logicInfo().getLogicString()
-       << ", which doesn't include " << tid
-       << ", but got a preprocessing-time fact for that theory." << endl
-       << "The fact:" << endl
-       << literal;
-    throw LogicException(ss.str());
-  }
+  // This should be implied by the check during ppRewrite, in particular
+  // literal should have been passed to ppRewrite.
+  Assert(isTheoryEnabled(d_env.theoryOf(atom))
+         || d_env.theoryOf(atom) == THEORY_SAT_SOLVER);
 
   Theory::PPAssertStatus solveStatus =
       theoryOf(atom)->ppAssert(tliteral, substitutionOut);
@@ -767,6 +762,20 @@ TrustNode TheoryEngine::ppRewrite(TNode term,
 {
   Assert(lems.empty());
   TheoryId tid = d_env.theoryOf(term);
+  // We check whether the theory is enabled here (instead of during solve),
+  // since there are corner cases where facts may involve terms that belong
+  // to other theories, e.g. equalities between variables belong to UF when
+  // theoryof-mode is `term`.
+  if (!isTheoryEnabled(tid) && tid != THEORY_SAT_SOLVER)
+  {
+    stringstream ss;
+    ss << "The logic was specified as " << logicInfo().getLogicString()
+       << ", which doesn't include " << tid
+       << ", but got a preprocessing-time term for that theory." << std::endl
+       << "The term:" << std::endl
+       << term;
+    throw LogicException(ss.str());
+  }
   TrustNode trn = d_theoryTable[tid]->ppRewrite(term, lems);
   // should never introduce a skolem to eliminate an equality
   Assert(lems.empty() || term.getKind() != kind::EQUAL);
@@ -1086,27 +1095,19 @@ theory::EqualityStatus TheoryEngine::getEqualityStatus(TNode a, TNode b)
   return d_sharedSolver->getEqualityStatus(a, b);
 }
 
-std::unordered_set<TNode> TheoryEngine::getRelevantAssertions(bool& success)
-{
-  // if we are not in SAT mode, or there is no relevance manager, we fail
-  if (!d_inSatMode || d_relManager == nullptr)
-  {
-    success = false;
-    // return empty set
-    return std::unordered_set<TNode>();
-  }
-  return d_relManager->getRelevantAssertions(success);
-}
-
 void TheoryEngine::getDifficultyMap(std::map<Node, Node>& dmap)
 {
   Assert(d_relManager != nullptr);
   d_relManager->getDifficultyMap(dmap);
 }
 
-theory::IncompleteId TheoryEngine::getIncompleteId() const
+theory::IncompleteId TheoryEngine::getModelUnsoundId() const
 {
-  return d_incompleteId.get();
+  return d_modelUnsoundId.get();
+}
+theory::IncompleteId TheoryEngine::getRefutationUnsoundId() const
+{
+  return d_refutationUnsoundId.get();
 }
 
 Node TheoryEngine::getModelValue(TNode var) {
@@ -1120,6 +1121,18 @@ Node TheoryEngine::getModelValue(TNode var) {
   return theoryOf(d_env.theoryOf(var.getType()))->getModelValue(var);
 }
 
+std::unordered_set<TNode> TheoryEngine::getRelevantAssertions(bool& success)
+{
+  // if there is no relevance manager, we fail
+  if (d_relManager == nullptr)
+  {
+    success = false;
+    // return empty set
+    return std::unordered_set<TNode>();
+  }
+  return d_relManager->getRelevantAssertions(success);
+}
+
 TrustNode TheoryEngine::getExplanation(TNode node)
 {
   Trace("theory::explain") << "TheoryEngine::getExplanation(" << node
@@ -1473,12 +1486,20 @@ void TheoryEngine::conflict(TrustNode tconflict, TheoryId theoryId)
   }
 }
 
-void TheoryEngine::setIncomplete(theory::TheoryId theory,
-                                 theory::IncompleteId id)
+void TheoryEngine::setModelUnsound(theory::TheoryId theory,
+                                   theory::IncompleteId id)
 {
-  d_incomplete = true;
-  d_incompleteTheory = theory;
-  d_incompleteId = id;
+  d_modelUnsound = true;
+  d_modelUnsoundTheory = theory;
+  d_modelUnsoundId = id;
+}
+
+void TheoryEngine::setRefutationUnsound(theory::TheoryId theory,
+                                        theory::IncompleteId id)
+{
+  d_refutationUnsound = true;
+  d_refutationUnsoundTheory = theory;
+  d_refutationUnsoundId = id;
 }
 
 TrustNode TheoryEngine::getExplanation(
