@@ -34,6 +34,7 @@
 #include "theory/rewriter.h"
 #include "theory/sort_inference.h"
 #include "theory/theory.h"
+#include "theory/theory_engine_statistics.h"
 #include "theory/theory_preprocessor.h"
 #include "theory/trust_substitutions.h"
 #include "theory/uf/equality_engine.h"
@@ -185,8 +186,16 @@ class TheoryEngine : protected EnvObj
   /** Notify (preprocessed) assertions. */
   void notifyPreprocessedAssertions(const std::vector<Node>& assertions);
 
-  /** Return whether or not we are incomplete (in the current context). */
-  bool isIncomplete() const { return d_incomplete; }
+  /**
+   * Return whether or not we are model unsound (in the current SAT context).
+   * For details, see theory_inference_manager.
+   */
+  bool isModelUnsound() const { return d_modelUnsound; }
+  /**
+   * Return whether or not we are refutation unsound (in the current user
+   * context). For details, see theory_inference_manager.
+   */
+  bool isRefutationUnsound() const { return d_refutationUnsound; }
 
   /**
    * Returns true if we need another round of checking.  If this
@@ -292,14 +301,13 @@ class TheoryEngine : protected EnvObj
   theory::TheoryModel* getModel();
   /**
    * Get the current model for the current set of assertions. This method
-   * should only be called immediately after a satisfiable or unknown
-   * response to a check-sat call, and only if produceModels is true.
+   * should only be called immediately after a satisfiable response to a
+   * check-sat call, and only if produceModels is true.
    *
-   * If the model is not already built, this will cause this theory engine
-   * to build the model.
+   * If the model is not already built, this will cause this theory engine to
+   * build the model.
    *
-   * If the model is not available (for instance, if the last call to check-sat
-   * was interrupted), then this returns the null pointer.
+   * If the model cannot be built, then this returns the null pointer.
    */
   theory::TheoryModel* getBuiltModel();
   /**
@@ -372,8 +380,10 @@ class TheoryEngine : protected EnvObj
    */
   void getDifficultyMap(std::map<Node, Node>& dmap);
 
-  /** Get incomplete id, valid immediately after an `unknown` response. */
-  theory::IncompleteId getIncompleteId() const;
+  /** Get incomplete id, valid when isModelUnsound is true. */
+  theory::IncompleteId getModelUnsoundId() const;
+  /** Get unsound id, valid when isRefutationUnsound is true. */
+  theory::IncompleteId getRefutationUnsoundId() const;
 
   /**
    * Forwards an entailment check according to the given theoryOfMode.
@@ -409,10 +419,10 @@ class TheoryEngine : protected EnvObj
   /** set in conflict */
   void markInConflict();
 
-  /**
-   * Called by the theories to notify that the current branch is incomplete.
-   */
-  void setIncomplete(theory::TheoryId theory, theory::IncompleteId id);
+  /** Called by the theories to notify that the current branch is incomplete. */
+  void setModelUnsound(theory::TheoryId theory, theory::IncompleteId id);
+  /** Called by the theories to notify that we are unsound (user-context). */
+  void setRefutationUnsound(theory::TheoryId theory, theory::IncompleteId id);
 
   /**
    * Called by the output channel to propagate literals and facts
@@ -521,9 +531,6 @@ class TheoryEngine : protected EnvObj
   /** The relevance manager */
   std::unique_ptr<theory::RelevanceManager> d_relManager;
 
-  /** are we in eager model building mode? (see setEagerModelBuilding). */
-  bool d_eager_model_building;
-
   /**
    * Output channels for individual theories.
    */
@@ -535,20 +542,21 @@ class TheoryEngine : protected EnvObj
   context::CDO<bool> d_inConflict;
 
   /**
-   * Are we in "SAT mode"? In this state, the user can query for the model.
-   * This corresponds to the state in Figure 4.1, page 52 of the SMT-LIB
-   * standard, version 2.6.
+   * True if a theory has notified us of model unsoundness (at this SAT
+   * context level or below). For details, see theory_inference_manager.
    */
-  bool d_inSatMode;
-
+  context::CDO<bool> d_modelUnsound;
+  /** The theory and identifier that (most recently) set model unsound */
+  context::CDO<theory::TheoryId> d_modelUnsoundTheory;
+  context::CDO<theory::IncompleteId> d_modelUnsoundId;
   /**
-   * True if a theory has notified us of incompleteness (at this
+   * True if a theory has notified us of refutation unsoundness (at this user
    * context level or below).
    */
-  context::CDO<bool> d_incomplete;
+  context::CDO<bool> d_refutationUnsound;
   /** The theory and identifier that (most recently) set incomplete */
-  context::CDO<theory::TheoryId> d_incompleteTheory;
-  context::CDO<theory::IncompleteId> d_incompleteId;
+  context::CDO<theory::TheoryId> d_refutationUnsoundTheory;
+  context::CDO<theory::IncompleteId> d_refutationUnsoundId;
 
   /**
    * Mapping of propagations from recievers to senders.
@@ -592,8 +600,8 @@ class TheoryEngine : protected EnvObj
   /** sort inference module */
   std::unique_ptr<theory::SortInference> d_sortInfer;
 
-  /** Time spent in theory combination */
-  TimerStat d_combineTheoriesTime;
+  /** Statistics */
+  theory::TheoryEngineStatistics d_stats;
 
   Node d_true;
   Node d_false;
