@@ -37,85 +37,101 @@ class Color:
     ENDC = "\033[0m"
 
 
+def print_info(msg):
+    print(Color.BLUE + "▶ " + msg + Color.ENDC)
+
+
+def print_ok(msg):
+    print(Color.GREEN + "✓ " + msg + Color.ENDC)
+
+
+def print_error(err):
+    print(Color.RED + "✖ " + err + Color.ENDC)
+
+
 class Tester:
-    def __init__(self):
-        pass
+
+    def __init__(self, name):
+        self.name = name
 
     def applies(self, benchmark_info):
         return True
 
-    def check_exit_status(self, name, expected_exit_status, exit_status, output, error, flags):
+    def check_exit_status(self, expected_exit_status, exit_status, output,
+                          error, flags):
         if exit_status == STATUS_TIMEOUT:
-            print("Timeout ({}) - Flags: {}".format(name, flags))
+            print_error("Timeout")
             return EXIT_SKIP if g_args.skip_timeout else EXIT_FAILURE
         elif exit_status != expected_exit_status:
-            print('not ok ({}) - Expected exit status "{}" but got "{}" - Flags: {}'.format(
-                name, expected_exit_status, exit_status, flags))
-            print()
-            print_outputs(output, error)
+            print_error(
+                'Unexpected exit status: expected "{}" but got "{}"'.format(
+                    expected_exit_status, exit_status))
             return EXIT_FAILURE
-        return None
+        return EXIT_OK
 
     def run(self, benchmark_info):
+        """Prints information about the test that is about to run and then
+        calls the internal run method to execute the test."""
+
+        print()
+        print_info(self.name)
+        print("  Flags: {}".format(benchmark_info.command_line_args))
+        return self.run_internal(benchmark_info)
+
+    def run_internal(self, benchmark_info):
+        """Runs cvc5 on a given benchmark and checks the output."""
+
         output, error, exit_status = run_benchmark(benchmark_info)
-        exit_code = self.check_exit_status(
-            "cvc5", benchmark_info.expected_exit_status, exit_status, output, error, benchmark_info.command_line_args)
-        if exit_code:
+        exit_code = self.check_exit_status(benchmark_info.expected_exit_status,
+                                           exit_status, output, error,
+                                           benchmark_info.command_line_args)
+        if exit_code == EXIT_SKIP:
             return exit_code
+
         if benchmark_info.compare_outputs and output != benchmark_info.expected_output:
-            print("not ok (cvc5) - Flags: {}".format(benchmark_info.command_line_args))
-            print()
-            print("Standard output difference")
-            print("=" * 80)
+            print_error("Unexpected output difference")
+            print("  " + "=" * 78)
             print_diff(output, benchmark_info.expected_output)
-            print("=" * 80)
-            print()
-            print()
-            if error:
-                print("Error output")
-                print_segment(Color.YELLOW, error)
-            return EXIT_FAILURE
+            print("  " + "=" * 78)
+            exit_code = EXIT_FAILURE
         if benchmark_info.compare_outputs and error != benchmark_info.expected_error:
-            print(
-                "not ok (cvc5) - Differences between expected and actual output on stderr - Flags: {}".format(
-                    benchmark_info.command_line_args
-                )
-            )
-            print()
-            print("Error output difference")
-            print("=" * 80)
+            print_error("Unexpected error output difference")
+            print("  " + "=" * 78)
             print_diff(error, benchmark_info.expected_error)
-            print("=" * 80)
-            print()
-            return EXIT_FAILURE
-        print("ok - Flags: {}".format(benchmark_info.command_line_args))
-        return EXIT_OK
+            print("  " + "=" * 78)
+            exit_code = EXIT_FAILURE
+        if exit_code == EXIT_OK:
+            print_ok("OK")
+        return exit_code
 
 
 ################################################################################
 # The testers
 #
-# Testers use `Tester` as a base class and implement `applies()` and `run()`
-# methods. The `applies()` method returns `True` if a tester applies to a given
-# benchmark and `run()` runs the actual test. Most testers can invoke the
-# `run()` method in the base class, which calls the cvc5 binary with a set of
-# arguments and checks the expected output (both stdout and stderr) as well as
-# the exit status.
+# Testers use `Tester` as a base class and implement `applies()` and
+# `run_internal()` methods. The `applies()` method returns `True` if a tester
+# applies to a given benchmark and `run_internal()` runs the actual test. Most
+# testers can invoke the `run_internal()` method in the base class, which calls
+# the cvc5 binary with a set of arguments and checks the expected output (both
+# stdout and stderr) as well as the exit status.
 #
 # To add a new tester, add a class and add it to the `g_tester` dictionary.
 ################################################################################
 
 
 class BaseTester(Tester):
-    def __init__(self):
-        pass
 
-    def run(self, benchmark_info):
-        return super().run(benchmark_info)
+    def __init__(self):
+        super().__init__("base")
+
+    def run_internal(self, benchmark_info):
+        return super().run_internal(benchmark_info)
 
 
 class UnsatCoreTester(Tester):
+
     def __init__(self):
+        super().__init__("unsat-core")
         pass
 
     def applies(self, benchmark_info):
@@ -129,8 +145,8 @@ class UnsatCoreTester(Tester):
             and "--unconstrained-simp" not in benchmark_info.command_line_args
         )
 
-    def run(self, benchmark_info):
-        return super().run(
+    def run_internal(self, benchmark_info):
+        return super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args
                 + ["--check-unsat-cores"]
@@ -139,8 +155,9 @@ class UnsatCoreTester(Tester):
 
 
 class ProofTester(Tester):
+
     def __init__(self):
-        pass
+        super().__init__("proof")
 
     def applies(self, benchmark_info):
         expected_output_lines = benchmark_info.expected_output.split()
@@ -149,8 +166,8 @@ class ProofTester(Tester):
             and "unsat" in benchmark_info.expected_output.split()
         )
 
-    def run(self, benchmark_info):
-        return super().run(
+    def run_internal(self, benchmark_info):
+        return super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args +
                 ["--check-proofs", "--proof-granularity=theory-rewrite", "--proof-check=lazy"]
@@ -159,13 +176,18 @@ class ProofTester(Tester):
 
 
 class LfscTester(Tester):
+
+    def __init__(self):
+        super().__init__("lfsc")
+
     def applies(self, benchmark_info):
         return (
             benchmark_info.benchmark_ext != ".sy"
             and benchmark_info.expected_output.strip() == "unsat"
         )
 
-    def run(self, benchmark_info):
+    def run_internal(self, benchmark_info):
+        exit_code = EXIT_OK
         with tempfile.NamedTemporaryFile() as tmpf:
             cvc5_args = benchmark_info.command_line_args + [
                 "--dump-proofs",
@@ -183,16 +205,15 @@ class LfscTester(Tester):
             tmpf.write(output.strip("unsat\n".encode()))
             tmpf.flush()
             output, error = output.decode(), error.decode()
-            exit_code = self.check_exit_status(
-                "cvc5", EXIT_OK, exit_status, output, error, cvc5_args)
-            if exit_code:
-                return exit_code
+            exit_code = self.check_exit_status(EXIT_OK, exit_status, output,
+                                               error, cvc5_args)
             if "check" not in output:
-                print(
-                    'not ok (cvc5) - Empty proof - Flags: {}'.format(exit_status, cvc5_args))
+                print_error("Empty proof")
                 print()
                 print_outputs(output, error)
                 return EXIT_FAILURE
+            if exit_code != EXIT_OK:
+                return exit_code
             output, error, exit_status = run_process(
                 [benchmark_info.lfsc_binary] +
                 benchmark_info.lfsc_sigs + [tmpf.name],
@@ -200,35 +221,36 @@ class LfscTester(Tester):
                 timeout=benchmark_info.timeout,
             )
             output, error = output.decode(), error.decode()
-            exit_code = self.check_exit_status(
-                "lfsc", EXIT_OK, exit_status, output, error, cvc5_args)
-            if exit_code:
-                return exit_code
+            exit_code = self.check_exit_status(EXIT_OK, exit_status, output,
+                                               error, cvc5_args)
             if "success" not in output:
-                print("not ok (lfsc) - Unexpected output - Flags: {}".format(cvc5_args))
+                print_error("Invalid proof")
                 print()
                 print_outputs(output, error)
                 return EXIT_FAILURE
-        print("ok - Flags: {}".format(cvc5_args))
-        return EXIT_OK
+        if exit_code == EXIT_OK:
+            print_ok("OK")
+        return exit_code
 
 
 class ModelTester(Tester):
+
     def __init__(self):
-        pass
+        super().__init__("model")
 
     def applies(self, benchmark_info):
         expected_output_lines = benchmark_info.expected_output.split()
-        return (
-            benchmark_info.benchmark_ext != ".sy"
-            and ("sat" in expected_output_lines or "unknown" in expected_output_lines)
-            and "--no-debug-check-models" not in benchmark_info.command_line_args
-            and "--no-check-models" not in benchmark_info.command_line_args
-            and "--debug-check-models" not in benchmark_info.command_line_args
-        )
+        return (benchmark_info.benchmark_ext != ".sy"
+                and ("sat" in expected_output_lines
+                     or "unknown" in expected_output_lines)
+                and "--no-debug-check-models"
+                not in benchmark_info.command_line_args
+                and "--no-check-models" not in benchmark_info.command_line_args
+                and "--debug-check-models"
+                not in benchmark_info.command_line_args)
 
-    def run(self, benchmark_info):
-        return super().run(
+    def run_internal(self, benchmark_info):
+        return super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args
                 + ["--debug-check-models"]
@@ -237,8 +259,9 @@ class ModelTester(Tester):
 
 
 class SynthTester(Tester):
+
     def __init__(self):
-        pass
+        super().__init__("synth")
 
     def applies(self, benchmark_info):
         return (
@@ -249,8 +272,8 @@ class SynthTester(Tester):
             and "--check-synth-sol" not in benchmark_info.command_line_args
         )
 
-    def run(self, benchmark_info):
-        return super().run(
+    def run_internal(self, benchmark_info):
+        return super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args
                 + ["--check-synth-sol"]
@@ -259,8 +282,9 @@ class SynthTester(Tester):
 
 
 class AbductTester(Tester):
+
     def __init__(self):
-        pass
+        super().__init__("abduct")
 
     def applies(self, benchmark_info):
         return (
@@ -270,8 +294,8 @@ class AbductTester(Tester):
             and "get-abduct" in benchmark_info.benchmark_content
         )
 
-    def run(self, benchmark_info):
-        return super().run(
+    def run_internal(self, benchmark_info):
+        return super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args + ["--check-abducts"]
             )
@@ -279,10 +303,14 @@ class AbductTester(Tester):
 
 
 class DumpTester(Tester):
+
+    def __init__(self):
+        super().__init__("dump")
+
     def applies(self, benchmark_info):
         return benchmark_info.benchmark_ext != ".p"
 
-    def run(self, benchmark_info):
+    def run_internal(self, benchmark_info):
         ext_to_lang = {
             ".smt2": "smt2",
             ".sy": "sygus",
@@ -311,7 +339,7 @@ class DumpTester(Tester):
         if not tmpf_name:
             return EXIT_FAILURE
 
-        exit_code = super().run(
+        exit_code = super().run_internal(
             benchmark_info._replace(
                 command_line_args=benchmark_info.command_line_args
                 + [
@@ -396,34 +424,37 @@ def print_colored(color, text):
 
 def print_segment(color, text):
     """Prints colored `text` inside a border."""
-    print("=" * 80)
+    print("  " + "=" * 78)
     for line in text.splitlines():
-        print(color + line + Color.ENDC)
-    print("=" * 80)
+        print("  " + color + line + Color.ENDC)
+    print("  " + "=" * 78)
     print()
 
 
 def print_outputs(stdout, stderr):
     """Prints standard output and error."""
-    print("Output:")
-    print_segment(Color.BLUE, stdout)
-    print()
-    print("Error output:")
-    print_segment(Color.YELLOW, stderr)
+    if stdout.strip() != "":
+        print("  Output:")
+        print_segment(Color.BLUE, stdout)
+    if stderr.strip() != "":
+        print("  Error output:")
+        print_segment(Color.YELLOW, stderr)
 
 
 def print_diff(actual, expected):
     """Prints the difference between `actual` and `expected`."""
 
-    for line in difflib.unified_diff(
-        expected.splitlines(), actual.splitlines(), "expected", "actual", lineterm=""
-    ):
+    for line in difflib.unified_diff(expected.splitlines(),
+                                     actual.splitlines(),
+                                     "expected",
+                                     "actual",
+                                     lineterm=""):
         if line.startswith("+"):
-            print_colored(Color.GREEN, line)
+            print_colored(Color.GREEN, "  " + line)
         elif line.startswith("-"):
-            print_colored(Color.RED, line)
+            print_colored(Color.RED, "  " + line)
         else:
-            print(line)
+            print("  " + line)
 
 
 def run_process(args, cwd, timeout, s_input=None):
@@ -435,8 +466,9 @@ def run_process(args, cwd, timeout, s_input=None):
     output and the exit code of the process. If the process times out, the
     output and the error output are empty and the exit code is 124."""
 
-    cmd = " ".join([shlex.quote(a) for a in args]) if isinstance(args, list) else args
-    print("Running command: {}".format(str(cmd)))
+    cmd = " ".join([shlex.quote(a)
+                    for a in args]) if isinstance(args, list) else args
+    print("  $ {}".format(str(cmd)))
     out = bytes()
     err = bytes()
     exit_status = STATUS_TIMEOUT
@@ -633,7 +665,8 @@ def run_regression(
         elif expected_exit_status is None:
             # If there is no expected output/error and the exit status has not
             # been set explicitly, the benchmark is invalid.
-            sys.exit('Cannot determine status of "{}"'.format(benchmark_path))
+            print_error('Cannot determine status of benchmark')
+            return EXIT_FAILURE
     if expected_exit_status is None:
         expected_exit_status = 0
 
@@ -646,20 +679,18 @@ def run_regression(
             req_feature = req_feature[len("no-") :]
             is_negative = True
         if req_feature not in (cvc5_features + cvc5_disabled_features):
-            print(
-                "Illegal requirement in regression: {}\nAllowed requirements: {}".format(
-                    req_feature, " ".join(cvc5_features + cvc5_disabled_features)
-                )
-            )
+            print_error(
+                "Illegal requirement in regression: {}".format(req_feature))
+            print("  Allowed requirements: {}".format(
+                " ".join(cvc5_features + cvc5_disabled_features)))
             return EXIT_FAILURE
         if is_negative:
             if req_feature in cvc5_features:
-                print(
-                    "1..0 # Skipped regression: not valid with {}".format(req_feature)
-                )
+                print_info("Skipped regression: not compatible with {}".format(
+                    req_feature))
                 return EXIT_SKIP if g_args.use_skip_return_code else EXIT_OK
         elif req_feature not in cvc5_features:
-            print("1..0 # Skipped regression: {} not supported".format(req_feature))
+            print_info("Skipped regression: requires {}".format(req_feature))
             return EXIT_SKIP if g_args.use_skip_return_code else EXIT_OK
 
     if not command_lines:
@@ -697,11 +728,9 @@ def run_regression(
                 tests.append((tester, benchmark_info))
 
     if len(tests) == 0:
-        print("1..0 # Skipped regression: no tests to run")
+        print("Skipped regression: no tests to run")
         return EXIT_SKIP if g_args.use_skip_return_code else EXIT_OK
 
-    print("1..{}".format(len(tests)))
-    print("# Starting")
     # Run cvc5 on the benchmark with the different testers and check whether
     # the exit status, stdout output, stderr output are as expected.
     exit_code = EXIT_OK
