@@ -43,8 +43,10 @@ TEST_F(TestApiBlackUncovered, exception_getmessage)
 TEST_F(TestApiBlackUncovered, streaming_operators)
 {
   std::stringstream ss;
-  ss << cvc5::modes::LEARNED_LIT_PREPROCESS;
   ss << cvc5::UnknownExplanation::UNKNOWN_REASON;
+  ss << cvc5::modes::BlockModelsMode::LITERALS;
+  ss << cvc5::modes::LearnedLitType::LEARNED_LIT_PREPROCESS;
+  ss << cvc5::modes::ProofComponent::PROOF_COMPONENT_FULL;
   ss << cvc5::Result();
   ss << cvc5::Op();
   ss << cvc5::SynthResult();
@@ -224,5 +226,27 @@ TEST_F(TestApiBlackUncovered, Statistics)
     testing::internal::GetCapturedStdout();
 }
 
+// Copied from api/cpp/solver_black.cpp
+TEST_F(TestApiBlackUncovered, declareOracleFunUnsat)
+{
+  d_solver.setOption("oracles", "true");
+  Sort iSort = d_solver.getIntegerSort();
+  // f is the function implementing (lambda ((x Int)) (+ x 1))
+  Term f = d_solver.declareOracleFun(
+      "f", {iSort}, iSort, [&](const std::vector<Term>& input) {
+        if (input[0].isUInt32Value())
+        {
+          return d_solver.mkInteger(input[0].getUInt32Value() + 1);
+        }
+        return d_solver.mkInteger(0);
+      });
+  Term three = d_solver.mkInteger(3);
+  Term five = d_solver.mkInteger(5);
+  Term eq =
+      d_solver.mkTerm(EQUAL, {d_solver.mkTerm(APPLY_UF, {f, three}), five});
+  d_solver.assertFormula(eq);
+  d_solver.checkSat();
+}
+
 }  // namespace test
 }  // namespace cvc5::internal
