@@ -43,7 +43,7 @@ class TestTheoryWhiteSequencesRewriter : public TestSmt
   {
     TestSmt::SetUp();
     Options opts;
-    d_rewriter = d_slvEngine->getRewriter();
+    d_rewriter = d_slvEngine->getEnv().getRewriter();
     d_seqRewriter.reset(new SequencesRewriter(d_rewriter, nullptr));
   }
 
